@@ -8,7 +8,7 @@ var (
 	defaultEsMemory     resource.Quantity = resource.MustParse("16Gi")
 	defaultEsCpuRequest resource.Quantity = resource.MustParse("1")
 
-	defaultEsProxyMemory     resource.Quantity = resource.MustParse("256Mi")
+	defaultEsProxyMemory     resource.Quantity = resource.MustParse("64Mi")
 	defaultEsProxyCpuRequest resource.Quantity = resource.MustParse("100m")
 
 	defaultKibanaMemory     resource.Quantity = resource.MustParse("736Mi")
@@ -22,7 +22,4 @@ var (
 
 	defaultFluentdMemory     resource.Quantity = resource.MustParse("736Mi")
 	defaultFluentdCpuRequest resource.Quantity = resource.MustParse("100m")
-
-	defaultPromTailMemory     resource.Quantity = resource.MustParse("358Mi")
-	defaultPromTailCpuRequest resource.Quantity = resource.MustParse("100m")
 )
